/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/** gclc:net.bigeon.gclc.command.ICommandProvider.java
 * Created on: Sep 6, 2014 */
package net.bigeon.gclc.command;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */

import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.InvalidCommandName;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;

/** An ICommadProvider is a provider of commands that can register commands
 * under some keywords.
 *
 * @author Emmanuel BIGEON */
public interface ICommandProvider {

    /** Adds a command to this provider, if no command was associated with the
     * given key.
     *
     * @param value the command to execute
     * @return if the command was added
     * @throws InvalidCommandName if the command name is invalid */
    boolean add(ICommand value) throws InvalidCommandName;

    /** Execute the command with the given name.
     * <p>
     * If no command with this name is found, an error command is usually
     * executed. If there are several commands with the same name, the behavior
     * is unspecified. Depending on the implementation, it may run an error
     * command or prompt the user for a choice.
     *
     * @param out the output
     * @param in the input
     * @param command the name of the command the user wishes to execute
     * @param args the arguments for the command
     * @throws CommandRunException if the command failed to run */
    void executeSub(ConsoleOutput out, ConsoleInput in, String command,
                    String... args) throws CommandRunException;

    /** Get the command with the given name.
     * <p>
     * If no command with this name is found, an error command is usually
     * returned. If there are several commands with the same name, the behavior
     * is unspecified. Depending on the implementation, it may return an error
     * command or the first command with this name found.
     *
     * @param command the name of the command the user wishes to execute
     * @return the command to execute */
    ICommand get(String command);

}
