/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.command.CommandTest.java
 * Created on: Nov 19, 2016
 */
package net.bigeon.gclc.command;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.junit.Test;

import net.bigeon.gclc.command.Command;
import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class CommandTest {

    @Test
    public final void testCommand() throws IOException {
        try (PipedConsoleOutput test = new PipedConsoleOutput()) {
            Command cmd;
            cmd = new Command("name") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return null;
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            };
            cmd.help(test);

            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // details
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "";
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            };
            cmd.help(test);
            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // details
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.Command#brief() */
                @Override
                protected String brief() {
                    return null;
                }

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "tip";
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            };
            cmd.help(test);
            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // details
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.Command#brief() */
                @Override
                protected String brief() {
                    return "brief";
                }

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "tip";
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            };
            cmd.help(test);
            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // details
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "tip";
                }

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.Command#usageDetail() */
                @Override
                protected String usageDetail() {
                    return null;
                }
            };
            cmd.help(test);
            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // details
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "tip";
                }

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.Command#usageDetail() */
                @Override
                protected String usageDetail() {
                    return "details";
                }
            };
            cmd.help(test);

            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // pattern
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // details
            assertEquals("Unexpected detail", "details", test.readNextLine());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "tip";
                }

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.Command#usageDetail() */
                @Override
                protected String usageDetail() {
                    return "details" + System.lineSeparator();
                }
            };
            cmd.help(test);
            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // pattern
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // details
            assertEquals("Unexpected detail", "details", test.readNextLine());
            assertFalse("Only 6 line should be printed", test.available());

            cmd = new Command("name") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    //
                }

                @Override
                public String tip() {
                    return "tip";
                }

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.Command#usageDetail() */
                @Override
                protected String usageDetail() {
                    return "\n";
                }
            };
            cmd.help(test);
            // name
            assertEquals("First line should be command name", "name",
                    test.readNextLine());
            // tip
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            // Usage
            test.readNextLine();
            // pattern
            test.readNextLine();
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            //
            assertTrue("Section line is empty", test.readNextLine().isEmpty());
            assertFalse("Only 6 line should be printed", test.available());

        }
    }
}
